/*
    TDMB412 driver for Linux -- kernel-interface.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Auxilliary functions that are required for kernel interface
    (i.e. those not found in file_operations structure)
*/

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <asm/io.h> 

#include "kernel-interface.h"
#include "tdmb412-const.h"
#include "tdmb412-cfg.h"
#include "tdmb412-vars.h"
#include "tdmb412-int.h"

/*
    tdmb412_init
        Reserve address space and IRQ's.  Initialize overflow buffers.  
        Return 1 on error, 0 on success.
*/

int tdmb412_init(void)

{
    int check_region_result ;	 /* result from check_region */
    int request_irq_result ;	 /* result from request_irq */

    printk("TDMB412 device driver module for Linux, version %s\n",
	        DRIVER_VERSION) ;

    #ifdef TRACE_INIT
        printk("[i+cr: ") ;
    #endif

    check_region_result = check_region(io_base, ADDRESS_EXTENT) ;

    if(check_region_result != 0)
    {
	   printk("Hardware address conflict at 0x%x, aborting module load\n", 
	           io_base) ;
	   return 1 ;
    }
    
    #ifdef TRACE_INIT
        printk("ok, ") ;
    #endif

    if(jtag_irq != 0)
    {
        #ifdef TRACE_INIT
            printk("jtag: ") ;
        #endif

	    request_irq_result = request_irq(jtag_irq, tdmb412_jtag_interrupt_handler, 
                                             0, DEVICE_NAME " (JTAG)", NULL) ;
	    if(request_irq_result != 0)
	    {
	        printk("IRQ%x (JTAG) is already in use, aborting module load\n",
		           jtag_irq) ;
	        return 1;
	    }
        #ifdef TRACE_INIT
            printk("ok, ") ;
        #endif
    }
    
    if(comport_irq != 0)
    {
        #ifdef TRACE_INIT
            printk("com: ") ;
        #endif

        request_irq_result = request_irq(comport_irq,
                                         tdmb412_comport_interrupt_handler, 0, 
                                         DEVICE_NAME " (COMPORT)", NULL) ;

        if(request_irq_result != 0)
        {
            printk("IRQ%x (COMPORT) is already in use, aborting module load\n",
                    comport_irq) ;

            if(jtag_irq != 0)
            {
                free_irq(jtag_irq, NULL) ;
            }

            return 1 ;
        } 
        #ifdef TRACE_INIT
            printk("ok, ") ;
        #endif
    }
    
    /*
       Reserve the IO space
    */

    #ifdef TRACE_INIT
        printk("iospace") ;
    #endif 
    
    request_region(io_base, ADDRESS_EXTENT, DEVICE_NAME) ;
    printk("%s: base address of tdmb412 is 0x%x\n", DEVICE_NAME, io_base) ;

    /*
       Write interrupt mask value to interrupt mask register if an
       interrupt for the comport interface was specified
    */

    #ifdef TRACE_INIT
	    printk(", int_mask") ;
    #endif

    if(comport_irq != 0)
    {
        outw(0, INT_MASK_REG) ;
        outw(INTERRUPT_MASK, INT_MASK_REG) ;
	    printk("%s: comport interface IRQ is %i\n", DEVICE_NAME,
                comport_irq) ;
    }

    else
    {
        printk("%s: comport interface IRQ is unspecified, disabling "
               "interrupt-driven I/O\n", DEVICE_NAME) ;
    }

    #ifdef TRACE_INIT
        printk("init_done.]\n") ;
    #endif
 
    printk("%s: compile time options: "
    #ifdef TRACE_INIT					
        "TRACE_INIT "				
    #endif							
    #ifdef WATCH_IOCTL					
        "WATCH_IOCTL "				
    #endif							
    #ifdef WATCH_OPEN_CLOSE					
        "WATCH_OPEN_CLOSE "				
    #endif							
    #ifdef WATCH_SEL					
        "WATCH_SEL "					
    #endif							
    #ifdef WATCH_RW						
        "WATCH_RW "					
    #endif							
    #ifdef WATCH_TIMEOUT					
        "WATCH_TIMEOUT "				
    #endif							
    #ifdef WATCH_XFER16_STATUS				
        "WATCH_XFER16_STATUS "			
    #endif							
    #ifdef WATCH_BURST_SIZE					
        "WATCH_BURST_SIZE "				
    #endif							
        "\n", DEVICE_NAME) ;

    return 0 ;
}
