/* 
    TDMB412 driver for Linux -- module-interface.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Contains code for functions specific to module-nature of the driver
*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/fs.h>

#include "kernel-interface.h"
#include "tdmb412-const.h"
#include "tdmb412-vars.h"

/*
    init_module 
        Called by kernel when the module is loaded.  Initialze the
        board (though tdmb412_init), then register the device with the
        kernel.  If anything goes wrong, let the user know and abort
        the loading of the module.
*/

int init_module(void)

{ 
    int tdmb412_init_result ;	 /* Result from tdmb412_init */
    int register_chrdev_result ; /* result from register_chrdev */

    tdmb412_init_result = tdmb412_init() ;

    if(tdmb412_init_result != 0)
    {
        return 1 ;
    }
    
    register_chrdev_result = register_chrdev(MAJOR_DEVICE_NUM, DEVICE_NAME, 
                                             &tdmb412_file_ops) ;

    if(register_chrdev_result != 0)      
    {
        printk("%s: unable to register device with kernel.  Change "
               "MAJOR_DEVICE_NUM in tdmb412-const.h to something that is not "
	           "already in use.\n", DEVICE_NAME) ;
        return 1 ;
    }
    
    printk("%s: module loaded successfully\n", DEVICE_NAME) ;

    return 0 ;
}

/*
    cleanup_module
        releases IRQs and board address space, then unregisters the driver 
        with the kernel
*/     
 
void cleanup_module(void)

{
    if(MOD_IN_USE)
    {
        printk("%s: device in use, module unload delayed\n", DEVICE_NAME) ;
    }

    else
    {
        if(jtag_irq != 0)
        {
            free_irq(jtag_irq, NULL) ;
        }

        if(comport_irq != 0)
        {
            free_irq(comport_irq, NULL) ;
        }

        release_region(io_base, ADDRESS_EXTENT) ;
        unregister_chrdev(MAJOR_DEVICE_NUM, DEVICE_NAME) ;

        printk("%s: module unloaded successfully\n", DEVICE_NAME) ;
    }
}
