/*
    TDMB412 driver for Linux -- tdmb412_ioctl.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    ioctl function for TDMB412.  Allows operations defined in
    tdmb412.h
*/

#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <asm/io.h>

#include "tdmb412-const.h"
#include "tdmb412.h"
#include "tdmb412-cfg.h"
#include "tdmb412-vars.h"
#include "tdmb412-ioctl.h"

/*
    tdmb412_ioctl
*/

int tdmb412_ioctl(struct inode *inode,	/* inode structure for device */
                  struct file *file,	/* file structure for device */
	          unsigned int cmd,     /* command (from tdmb412.h)*/
	          unsigned long arg)	/* Argument to command (if any) */
{
    int delay_count ;		            /* To implement delay for reset */

    switch(cmd)
    {
        case TDMB412_RESET:
            #ifdef WATCH_IOCTL
                printk("ioctl(TDMB412_RESET) ");
            #endif
            down(&device_mutex) ;
            outb(0x01, RESET_REG) ;
            delay_count = jiffies + 1 ;
            while(jiffies < delay_count) ;
            outb(0x00, RESET_REG) ;
            up(&device_mutex) ;
            return 0 ;
            break ;

        case TDMB412_8_BIT_XFERS:
        case TDMB412_16_BIT_XFERS:
            #ifdef WATCH_IOCTL
                printk("ioctl(TDMB412_x_BIT_XFERS) ");
            #endif
            down(&var_mutex) ;
            transfer_mode = cmd ;
            up(&var_mutex) ;
            return 0 ;
            break ;
	
        case LINKREADABORT:	    /* For compatability with transputer-08 */
        case LINKWRITEABORT:	    /* driver */
            #ifdef WATCH_IOCTL
                printk("ioctl(LINKxxxABORT) ");
            #endif
            return 0;
            break;

        default:
            printk("%s (%s:%i): illegal ioctl command detected- ignoring\n",
                    DEVICE_NAME, __FILE__, __LINE__) ;
            return -EPERM ;
    }
}
