/*
    TDMB412 driver for Linux -- tdmb412-open-close.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Implementation for open and close (release) for TDMB412 driver
*/

#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/sched.h>

#include "tdmb412-cfg.h"
#include "tdmb412-vars.h"
#include "tdmb412-open-close.h"

/*
    tdmb412_open
        Opens the device (or, at least, marks it as opened).  Complains
        if it's alrady opened.
*/

int tdmb412_open(struct inode *inode,	/* inode structure for device */
                 struct file *file)	    /* file structure for device */

{
    #ifdef WATCH_OPEN_CLOSE
        printk("o");
    #endif
    down(&var_mutex) ;

    if(opened != 0)
    {
        up(&var_mutex) ;

        #ifdef WATCH_OPEN_CLOSE
	        printk("- ");
        #endif

        return -EBUSY ;
    }

    else
    {
	    opened = 1 ;
        #ifdef WATCH_OPEN_CLOSE
	        printk("+ ");
        #endif
	    up(&var_mutex) ;
	    return 0 ;
    }
}

/*
    tdmb412_release
        Releases/closes the device (again, just changes a flag).
*/

void tdmb412_release(struct inode *inode, /* inode structure for device */
		             struct file *file)	  /* file structure for device */

{
    #ifdef WATCH_OPEN_CLOSE
        printk("c");
    #endif
    down(&var_mutex) ;
    opened = 0 ;
    up(&var_mutex) ;
}
