/*
    TDMB412 Driver for Linux -- tdmb412-read.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Contains implementations of read and associated functions
*/

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <asm/io.h>

#include "tdmb412-const.h"
#include "tdmb412.h"
#include "tdmb412-cfg.h"
#include "tdmb412-read.h"
#include "tdmb412-vars.h"
#include "tdmb412-rbuf.h"
#include "tdmb412-xfer.h"

/*
    Local variables
*/

unsigned long flags ;		      /* CPU flags */

/*
    tdmb412_read
        Reads data from the device in the current mode of operation
        (default is specified in DEFAULT_TRANSFER_MODE, can be changed
        through ioctl call)
*/

int tdmb412_read(struct inode *inode,    /* Inode struct for special file */
                 struct file *file,      /* File structure for device */
                 char *user_buffer,      /* Data buffer (user space) */
                 int user_buffer_size)   /* Size of data buffer */

{
    #ifdef WATCH_RW
        printk("[r");
    #endif

    if(user_buffer_size == 0)
    {
        #ifdef WATCH_RW
            printk("]") ;
        #endif
        return 0 ;
    }

    if((user_buffer_size % 4) != 0)
    {
        #ifdef WATCH_RW
	        printk("-]");
        #endif
	    return -EINVAL ;
    }

    #ifdef WATCH_RW
        printk("+");
    #endif

    /*
    if(comport_irq != 0)
    {
	    return get_data_from_read_buf(user_buffer, user_buffer_size) ;
    }

    else
    {
        */
	    return get_data_from_device(user_buffer, user_buffer_size) ;
    /* } */
}

/*
    get_data_from_device
        Performs the actual reading of the data from the device into
        the given buffer.  Returns the number of bytes read.
*/

int get_data_from_device(char *user_buffer,	    /* Buffer to read into */
			             int size)              /* Size of buffer */

{
    int bytes_read;		/* Number of bytes read */

    down(&device_mutex) ;
    down(&var_mutex) ;

    if( (transfer_mode != TDMB412_8_BIT_XFERS) && 
       (transfer_mode != TDMB412_16_BIT_XFERS) )
    {
	    printk("%s (%s:%i): invalid transfer mode detected, forcing %i"
	           "bit xfers\n", DEVICE_NAME, __FILE__, __LINE__,
	           DEFAULT_TRANSFER_MODE == TDMB412_8_BIT_XFERS ? 8 : 16) ;
	    transfer_mode = DEFAULT_TRANSFER_MODE ;
    }


    switch(transfer_mode) 
    {
        case TDMB412_16_BIT_XFERS:
            bytes_read = get_16bit_data(user_buffer, size) ;
            break ;

        case TDMB412_8_BIT_XFERS:
            bytes_read = get_8bit_data(user_buffer, size) ;
            break ;

        default:
            bytes_read = 0;
    }

    up(&var_mutex) ;
    up(&device_mutex) ; 

    #ifdef WATCH_RW
        printk("]");
    #endif

    return bytes_read ;
}
