/*
    TDMB412 driver for Linux -- tdmb412_select.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Select function for TDMB412.  
*/

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <asm/io.h>

#include "tdmb412-const.h"
#include "tdmb412.h"
#include "tdmb412-cfg.h"
#include "tdmb412-vars.h"
#include "tdmb412-sel.h"

/*
    Local variables
*/

static struct wait_queue *sel_wait_queue ;

/*
   tdmb412_select
*/

int tdmb412_select(struct inode *inode, /* inde structure for device */
                   struct file *file,	/* file stucture for device */
                   int sel_type,	    /* select type (read, write, ex) */
                   select_table *wait)	/* Used by select_wait */

{
    #ifdef WATCH_SEL
        printk("sel_ ");
    #endif

    switch(sel_type)
    {
        case SEL_IN:
            return handle_sel_in(wait) ;
            return 0 ;
            break ;
	
        case SEL_OUT:
            return handle_sel_out(wait) ;
            return 0 ;
            break ;

        case SEL_EX:
            return handle_sel_ex(wait) ;
            return 0 ;
            break ;

        default:
            printk("%s (%s:%i): illegal sel_type detected- ignoring\n",
                DEVICE_NAME, __FILE__, __LINE__) ;
            return -EPERM ;
    }	
}

/*
    handle_sel_in
       If the card has data available for reading, return 1, otherwise
       call select_wait() and return 0
*/

int handle_sel_in(select_table *wait)

{
    int status ;		    /* Status from card */

    #ifdef WATCH_SEL
        printk("in ");
    #endif

    down(&device_mutex) ;	/* do these in alphabetical order */
    down(&var_mutex) ;

    switch(transfer_mode)
    {
        case TDMB412_8_BIT_XFERS:

            status = inb(RXSTAT_8_REG) ;
	
            if(status)
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                return 1 ;
            }
            else
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                select_wait(&sel_wait_queue, wait) ;
                return 0 ;
            }

            break ;

        case TDMB412_16_BIT_XFERS:
               
	    status = inw(RXSTAT_16_REG) ;

    #ifdef WATCH_SEL
        printk("rxstat-16-reg %x ",status);
    #endif

            if(status & RXSTAT_16_DP)
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                return 1 ;
            }
            else
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                select_wait(&sel_wait_queue, wait) ;
                return 0 ;
            }
            break ;

        default:
            printk("%s (%s:%i): invalid transfer mode detected, forcing %i"
                "bit xfers\n", DEVICE_NAME, __FILE__, __LINE__,
                DEFAULT_TRANSFER_MODE == TDMB412_8_BIT_XFERS ? 8 : 16) ;
            transfer_mode = DEFAULT_TRANSFER_MODE ;
            up(&var_mutex) ;
            up(&device_mutex) ;

            /*
                recurse.  I don't feel like writing a loop and gotos are
                bad
            */

            return handle_sel_in(wait) ;
    }
}

/*
   handle_sel_out
*/

int handle_sel_out(select_table *wait) 

{
    int status ;		/* Status from card */

    #ifdef WATCH_SEL
        printk("out ");
    #endif

    down(&device_mutex) ;	/* do these in alphabetical order */
    down(&var_mutex) ;

    switch(transfer_mode)
    {
        case TDMB412_8_BIT_XFERS:
            status = inb(TXSTAT_8_REG) ;
            
            if(status)
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                return 1 ;
            }

            else
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                select_wait(&sel_wait_queue, wait) ;
                return 0 ;
            }

            break ;

        case TDMB412_16_BIT_XFERS:
            status = inw(TXSTAT_16_REG) ;

            if(status & TXSTAT_16_OR)
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                return 1 ;
            }

            else
            {
                up(&var_mutex) ;
                up(&device_mutex) ;
                select_wait(&sel_wait_queue, wait) ;
                return 0 ;
            }

            break ;

        default:
            printk("%s (%s:%i): invalid transfer mode detected, forcing %i"
                "bit xfers\n", DEVICE_NAME, __FILE__, __LINE__,
                DEFAULT_TRANSFER_MODE == TDMB412_8_BIT_XFERS ? 8 : 16) ;
            transfer_mode = DEFAULT_TRANSFER_MODE ;
            up(&var_mutex) ;
            up(&device_mutex) ;

            /*
                recurse.  I don't feel like writing a loop and gotos are
                bad
            */

            return handle_sel_out(wait) ;
    }
}

/*
    handle_sel_ex
        Do do anything hear just yet- we aren't supporting exceptions.
*/

int handle_sel_ex(select_table *wait) 

{
    #ifdef WATCH_SEL
        printk("wait ");
    #endif

    select_wait(&sel_wait_queue, wait) ;
    return 0 ;
}
