/*
    TDMB412 Driver for Linux -- tdmb412-write.c
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    Contains implementations of read and associated functions
*/

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <asm/io.h>

#include "tdmb412-const.h"
#include "tdmb412.h"
#include "tdmb412-cfg.h"
#include "tdmb412-xfer.h"
#include "tdmb412-write.h"
#include "tdmb412-vars.h"

/*
    Local variables
*/

unsigned long flags ;		      /* CPU flags */

/*
    tdmb412_write
        Writes data from the device in the current mode of operation
	    (default is specified in DEFAULT_TRANSFER_MODE, can be changed
	    through ioctl call)
*/

int tdmb412_write(struct inode *inode,             /* Inode struct for special file */
		          struct file *file,       /* File structure for device */
		          const char *user_buffer, /* Data buffer (user space) */
		          int user_buffer_size)    /* Size of data buffer */

{
    #ifdef WATCH_RW
        printk("[w");
    #endif

    if(user_buffer_size == 0)
    {
        #ifdef WATCH_RW
            printk("]") ;
        #endif
        return 0 ;
    }

    if((user_buffer_size % 4) != 0)
    {
        #ifdef WATCH_RW
	        printk("-]");
        #endif
	    return -EINVAL ;
    }

    #ifdef WATCH_RW
        printk("+");
    #endif
    
    return put_data_to_device(user_buffer, user_buffer_size) ;
}

/*
    put_data_to_device
        Performs the actual writing of the data to the device from
        the given buffer.  Returns the number of bytes written.
*/

int put_data_to_device(const char *user_buffer,    /* Buffer to read into */
		       int size)                   /* Size of buffer */

{
    int bytes_written ;		                  /* Number of bytes written */
    
    down(&device_mutex) ;
    down(&var_mutex) ;

    if( (transfer_mode != TDMB412_8_BIT_XFERS) && 
       (transfer_mode != TDMB412_16_BIT_XFERS) )
    {
        printk("%s (%s:%i): invalid transfer mode detected, forcing %i"
            "bit xfers\n", DEVICE_NAME, __FILE__, __LINE__,
            DEFAULT_TRANSFER_MODE == TDMB412_8_BIT_XFERS ? 8 : 16) ;
        transfer_mode = DEFAULT_TRANSFER_MODE ;
    }

    switch(transfer_mode) 
    {
        case TDMB412_16_BIT_XFERS :
            bytes_written = put_16bit_data(user_buffer, size) ;
            break ;
	
        case TDMB412_8_BIT_XFERS:
            bytes_written = put_8bit_data(user_buffer, size) ;
            break ;

        default:
            bytes_written = 0 ;
    }

    up(&var_mutex) ;
    up(&device_mutex) ;

    #ifdef WATCH_RW
        printk("]");
    #endif

    return bytes_written ;
}
